@@ -28,7 +28,7 @@
 
 #include "gossip-transport-discover.h"
 
-#define d(x) x
+#define d(x)
 
 /* in seconds */
 #define DISCO_TIMEOUT      20
@@ -136,7 +136,7 @@ transport_disco_new (GossipJabber *jabber)
 
 	disco->jabber = g_object_ref (jabber);
 
-	connection = _gossip_jabber_get_connection (jabber);
+	connection = gossip_jabber_get_connection (jabber);
 
 	handler = lm_message_handler_new (transport_disco_message_handler, disco, NULL);
 	disco->message_handler = handler;
@@ -371,7 +371,7 @@ transport_disco_request_items (GossipTransportDisco *disco)
 	d(g_print ("disco items to: %s\n", 
 		   gossip_jid_get_full (disco->to)));
 	
-	connection = _gossip_jabber_get_connection (disco->jabber);
+	connection = gossip_jabber_get_connection (disco->jabber);
 
 	lm_message_node_add_child (m->node, "query", NULL);
 	node = lm_message_node_get_child (m->node, "query");
@@ -486,7 +486,7 @@ transport_disco_request_info (GossipTransportDisco *disco)
 	GList        *l;
 	GossipJID    *jid;
 	
-	connection = _gossip_jabber_get_connection (disco->jabber);
+	connection = gossip_jabber_get_connection (disco->jabber);
 	jid = gossip_jid_new ("users.jabber.org");
 
 	disco->items_total = disco->items_remaining = g_list_length (disco->items);
@@ -695,7 +695,7 @@ gossip_transport_disco_destroy (GossipTransportDisco *disco)
 
 	disco->destroying = TRUE;
 
-        connection = _gossip_jabber_get_connection (disco->jabber);
+        connection = gossip_jabber_get_connection (disco->jabber);
 
         handler = disco->message_handler;
         if (handler) {
@@ -754,7 +754,7 @@ gossip_transport_disco_request_info (GossipJabber                 *jabber,
 	disco->jabber = g_object_ref (jabber);
 
 	/* set up handler */
-	connection = _gossip_jabber_get_connection (jabber);
+	connection = gossip_jabber_get_connection (jabber);
 
 	handler = lm_message_handler_new (transport_disco_message_handler, disco, NULL);
 	disco->message_handler = handler;
