@@ -29,19 +29,22 @@
 
 #include <libgossip/gossip-protocol.h>
 
+#include "gossip-add-contact-window.h"
 #include "gossip-app.h"
-#include "gossip-contact-info.h"
+#include "gossip-chat-invite.h"
+#include "gossip-chat-window.h"
+#include "gossip-contact-info-dialog.h"
+#include "gossip-log.h"
+#include "gossip-notebook.h"
 #include "gossip-preferences.h"
 #include "gossip-private-chat.h"
-#include "gossip-chat-window.h"
-#include "gossip-notebook.h"
-#include "gossip-stock.h"
 #include "gossip-sound.h"
-#include "gossip-log.h"
+#include "gossip-stock.h"
 #include "gossip-ui-utils.h"
-#include "eel-ellipsizing-label.h"
 
 
+#define d(x)
+
 static void       gossip_chat_window_class_init        (GossipChatWindowClass *klass);
 static void       gossip_chat_window_init              (GossipChatWindow      *window);
 static void       gossip_chat_window_finalize          (GObject               *object);
@@ -63,6 +66,8 @@ static void       chat_window_clear_activate_cb        (GtkWidget             *m
 							GossipChatWindow      *window);
 static void       chat_window_info_activate_cb         (GtkWidget             *menuitem,
 							GossipChatWindow      *window);
+static void       chat_window_add_contact_activate_cb  (GtkWidget             *menuitem,
+							GossipChatWindow      *window);
 static void       chat_window_log_activate_cb          (GtkWidget             *menuitem,
 							GossipChatWindow      *window);
 static void       chat_window_conv_activate_cb         (GtkWidget             *menuitem,
@@ -146,6 +151,7 @@ struct _GossipChatWindowPriv {
 
 	/* Menu items. */
 	GtkWidget   *m_conv_clear;
+	GtkWidget   *m_conv_add_contact;
 	GtkWidget   *m_conv_log;
 	GtkWidget   *m_conv_info;
 	GtkWidget   *s_conv_show_contacts;
@@ -216,8 +222,9 @@ gossip_chat_window_init (GossipChatWindow *window)
 				       "chats_notebook", &priv->notebook,
 				       "menu_conv", &menu_conv,
 				       "menu_conv_clear", &priv->m_conv_clear,
-				       "menu_conv_info", &priv->m_conv_info,
+				       "menu_conv_add_contact", &priv->m_conv_add_contact,
 				       "menu_conv_log", &priv->m_conv_log,
+				       "menu_conv_info", &priv->m_conv_info,
 				       "sep_conv_show_contacts", &priv->s_conv_show_contacts,
 				       "menu_conv_show_contacts", &priv->m_conv_show_contacts,
 				       "sep_conv_invite_contacts", &priv->s_conv_invite_contacts,
@@ -258,6 +265,10 @@ gossip_chat_window_init (GossipChatWindow *window)
 			  "activate",
 			  G_CALLBACK (chat_window_clear_activate_cb),
 			  window);
+	g_signal_connect (priv->m_conv_add_contact,
+			  "activate",
+			  G_CALLBACK (chat_window_add_contact_activate_cb),
+			  window);
 	g_signal_connect (priv->m_conv_log,
 			  "activate",
 			  G_CALLBACK (chat_window_log_activate_cb),
@@ -361,69 +372,53 @@ gossip_chat_window_init (GossipChatWindow *window)
 /* Returns the window to open a new tab in if there is only one window visble */
 /* Otherwise, returns NULL indicating that a new window should be added       */
 GossipChatWindow *
-gossip_chat_window_get_default (gboolean for_group_chats)
+gossip_chat_window_get_default (void)
 {
-	GossipChatWindow *default_chat_window = NULL;
-	GList            *l;
+	GList       *l;
+        gboolean     separate_windows = TRUE;
+        GConfClient *gconf;
+        GError      *error;
+        
+        gconf = gconf_client_get_default ();
 
-	for (l=chat_windows; l; l=l->next) {
+        error = NULL;
+        separate_windows = gconf_client_get_bool (gconf,
+                                                  "/apps/gossip/ui/separate_chat_windows",
+                                                  &error);
+        if (error) {
+                separate_windows = FALSE;
+                g_error_free (error);
+        }
+
+        if (separate_windows) {
+                /* Always create a new window */
+                return NULL;
+        }
+
+	for (l = chat_windows; l; l = l->next) {
 		GossipChatWindow *chat_window;
 		GtkWidget        *dialog;
 		GdkWindow        *window;
 		gboolean          visible;
-		gint              count;
 
 		chat_window = l->data;
-
-		count = gossip_chat_window_get_group_chats (chat_window);
-		if ((count < 1 && for_group_chats) || (count > 0 && !for_group_chats)) {
-			continue;
-		}
-
+                
 		dialog = gossip_chat_window_get_dialog (chat_window);
-		window = dialog->window;
+                window = dialog->window;
 
 		g_object_get (dialog, 
 			      "visible", &visible,
 			      NULL);
 		
 		visible = visible && !(gdk_window_get_state (window) & GDK_WINDOW_STATE_ICONIFIED);
-
-		/* more than one visible window */
-		if (visible && default_chat_window) {
-			return NULL;
-		}
-
-		if (visible) {
-			default_chat_window = chat_window;
-		}
+                
+                if (visible) {
+                        /* Found a visible window on this desktop */
+                        return chat_window;
+                }
 	}
 
-	return default_chat_window;
-}
-
-gint 
-gossip_chat_window_get_group_chats (GossipChatWindow *window)
-{
-	GossipChatWindowPriv *priv;
- 	GList                *l;
-	gint                  count;
-
-	g_return_val_if_fail (window != NULL, 0);
-
-	priv = window->priv;
-	
-	for (l=priv->chats, count=0; l; l=l->next) {
-		GossipChat *chat;
-
-		chat = GOSSIP_CHAT (l->data);
-
-		if (gossip_chat_get_group_chat (chat)) {
-			count++;
-		}
-	}
-
-	return count;
+        return NULL;
 }
 
 static void
@@ -452,18 +447,20 @@ chat_window_get_status_pixbuf (GossipChatWindow *window,
 	session = gossip_app_get_session ();
 
 	if (g_list_find (window->priv->chats_new_msg, chat)) {
-		pixbuf = gossip_ui_utils_get_pixbuf_from_stock (GOSSIP_STOCK_MESSAGE);
+		pixbuf = gossip_pixbuf_from_stock (GOSSIP_STOCK_MESSAGE, 
+						   GTK_ICON_SIZE_MENU);
 	}
 	else if (g_list_find (window->priv->chats_composing, chat)) {
-		pixbuf = gossip_ui_utils_get_pixbuf_from_stock (GOSSIP_STOCK_TYPING);
+		pixbuf = gossip_pixbuf_from_stock (GOSSIP_STOCK_TYPING, 
+						   GTK_ICON_SIZE_MENU);
 	}
-	else if (!gossip_session_is_connected (session)) {
+	else if (!gossip_session_is_connected (session, NULL)) {
 		/* FIXME: should use protocol for contact since the
 		   protocol we use to send the image may not be online
 		   and a session may cover multiple accounts */
 
 		/* always offline if disconnected */
-		pixbuf = gossip_ui_utils_get_pixbuf_offline ();
+		pixbuf = gossip_pixbuf_offline ();
 	}
 	else {
 		pixbuf = gossip_chat_get_status_pixbuf (chat);
@@ -575,10 +572,17 @@ chat_window_update_title (GossipChatWindow *window)
 
 	gtk_window_set_title (GTK_WINDOW (window->priv->dialog), title);
 	if (window->priv->new_msg) {
-		pixbuf = gossip_ui_utils_get_pixbuf_from_stock (GOSSIP_STOCK_MESSAGE);
+		pixbuf = gossip_pixbuf_from_stock (GOSSIP_STOCK_MESSAGE,
+						   GTK_ICON_SIZE_MENU);
 		gtk_window_set_icon (GTK_WINDOW (window->priv->dialog), pixbuf);
+#if HAVE_GTK_URGENCY
+		gtk_window_set_urgency_hint (GTK_WINDOW (window->priv->dialog), TRUE);
+#endif
         } else {
 		gtk_window_set_icon (GTK_WINDOW (window->priv->dialog), NULL);
+#if HAVE_GTK_URGENCY
+		gtk_window_set_urgency_hint (GTK_WINDOW (window->priv->dialog), FALSE);
+#endif
 	}
 	g_free (title);
 }
@@ -640,101 +644,78 @@ chat_window_update_menu (GossipChatWindow *window)
 						   chat_window_show_contacts_toggled_cb,
 						   window);
 	} else {
+		GossipContactType type;
+
 		gtk_widget_hide (priv->s_conv_show_contacts);
 		gtk_widget_hide (priv->m_conv_show_contacts);
+
+		type = gossip_contact_get_type (contact);
+		if (type != GOSSIP_CONTACT_TYPE_CONTACTLIST &&
+		    type != GOSSIP_CONTACT_TYPE_USER) {
+			gtk_widget_show (priv->m_conv_add_contact);
+		} else {
+			gtk_widget_hide (priv->m_conv_add_contact);
+		}
 	}
 
-	chat_window_invite_menu_setup (window); 
+ 	chat_window_invite_menu_setup (window);  
 }
 
 static void
-chat_window_invite_menu_setup (GossipChatWindow *window)
+chat_window_invite_menu_setup (GossipChatWindow *window) 
 {
-	GossipChatWindowPriv   *priv;
-	GossipSession          *session;
-	GossipChatroomProvider *provider;
-	GList                  *list = NULL;
-
-	gboolean                is_group_chat;
+	GossipChatWindowPriv *priv;
+	GossipContact        *contact;
+	GtkWidget            *menu = NULL;
+	gboolean              is_group_chat;
 
 	priv = window->priv;
+	
+	contact = gossip_chat_get_own_contact (priv->current_chat);
+	
+	if (!contact) {
+		/* use the other contact if we don't have ourselves */
+		contact = gossip_chat_get_contact (priv->current_chat);
 
-	session = gossip_app_get_session ();
-	provider = gossip_session_get_chatroom_provider (session);
+		/* FIXME: Don't know if this is correct, but it fixes two
+		 * critical warnings when joining a group chat.
+		 */
+		if (!contact) {
+			return;
+		}
+	}
 
 	is_group_chat = gossip_chat_get_group_chat (priv->current_chat);
 
 	if (is_group_chat) {
-		GossipProtocol *protocol = NULL;
-		const GList    *contacts;
-		const GList    *l;
+		GossipChatroomId id;
 
-		contacts = gossip_protocol_get_contacts (protocol);
-		for (l = contacts; l; l = l->next) {
-			GossipContact *contact;
-
-			contact = l->data;
-
-			if (!gossip_contact_is_online (contact)) {
-				continue;
-			}
-
-			list = g_list_append (list, contact);
-		}
+		id = gossip_group_chat_get_room_id (GOSSIP_GROUP_CHAT (priv->current_chat));
+		menu = gossip_chat_invite_groupchat_menu (contact, id);
 	} else {
-		list = gossip_chatroom_provider_get_rooms (provider);
+ 		menu = gossip_chat_invite_contact_menu (contact); 
 	}
 
-	if (0) {
-/* 	if (g_list_length (list) > 0) { */
-		GtkWidget *rooms_menu;
-		GList     *l;
-
+	if (menu) {
 		gtk_widget_show (priv->s_conv_invite_contacts);
 		gtk_widget_show (priv->m_conv_invite_contacts);
 
-		rooms_menu = gtk_menu_new ();
+		gtk_widget_show_all (menu);
+		
+		gtk_menu_item_set_submenu (GTK_MENU_ITEM (priv->m_conv_invite_contacts),
+					   menu);
 
-		for (l = list; l; l = l->next) {
-			GossipChatroomId  id;
-			GossipContact    *contact;
-			const gchar      *name;
-			GtkWidget        *item;
-
-			if (is_group_chat) {
-				contact = l->data;
-				name = gossip_contact_get_name (contact);
-			} else {
-				id = GPOINTER_TO_INT(l->data);
-				name = gossip_chatroom_provider_get_room_name (provider, id);
-			}
-
-			if (!name) {
-				continue;
-			}
-
-			item = gtk_menu_item_new_with_label (name);
-			gtk_menu_shell_append (GTK_MENU_SHELL (rooms_menu), item);
-			
-			gtk_widget_show_all (rooms_menu);
-			
-			gtk_menu_item_set_submenu (GTK_MENU_ITEM (priv->m_conv_invite_contacts),
-						   rooms_menu);
-		}
 	} else {
+		gtk_menu_item_remove_submenu (GTK_MENU_ITEM (priv->m_conv_invite_contacts));
+
 		gtk_widget_hide (priv->s_conv_invite_contacts);
 		gtk_widget_hide (priv->m_conv_invite_contacts);
-
-		gtk_menu_item_remove_submenu (GTK_MENU_ITEM (priv->m_conv_invite_contacts));
-	}
-
-	if (is_group_chat) {
-		g_list_free (list);
 	}
 }
 
 static void
-chat_window_clear_activate_cb (GtkWidget *menuitem, GossipChatWindow *window)
+chat_window_clear_activate_cb (GtkWidget        *menuitem, 
+			       GossipChatWindow *window)
 {
 	GossipChatWindowPriv *priv = window->priv;
 
@@ -742,18 +723,32 @@ chat_window_clear_activate_cb (GtkWidget *menuitem, GossipChatWindow *window)
 }
 
 static void
-chat_window_log_activate_cb (GtkWidget *menuitem, GossipChatWindow *window)
+chat_window_add_contact_activate_cb (GtkWidget        *menuitem, 
+				     GossipChatWindow *window)
 {
 	GossipChatWindowPriv *priv = window->priv;
 	GossipContact        *contact;
 
 	contact = gossip_chat_get_contact (priv->current_chat);
-	
-	gossip_log_show (contact);
+
+	gossip_add_contact_window_show (NULL, contact);
 }
 
 static void
-chat_window_info_activate_cb (GtkWidget *menuitem, GossipChatWindow *window)
+chat_window_log_activate_cb (GtkWidget        *menuitem, 
+			     GossipChatWindow *window)
+{
+	GossipChatWindowPriv *priv = window->priv;
+	GossipContact        *contact;
+
+	contact = gossip_chat_get_contact (priv->current_chat);
+
+	gossip_log_show (GTK_WIDGET (priv->dialog), contact);
+}
+
+static void
+chat_window_info_activate_cb (GtkWidget        *menuitem, 
+			      GossipChatWindow *window)
 {
 	GossipChatWindowPriv *priv;
 	GossipContact        *contact;
@@ -762,7 +757,7 @@ chat_window_info_activate_cb (GtkWidget *menuitem, GossipChatWindow *window)
 
 	contact = gossip_chat_get_contact (priv->current_chat);
 
-	gossip_contact_info_new (contact);
+	gossip_contact_info_dialog_show (contact);
 }
 
 static void
@@ -812,7 +807,7 @@ chat_window_close_activate_cb (GtkWidget        *menuitem,
 
 static void
 chat_window_cut_activate_cb (GtkWidget        *menuitem,
-			      GossipChatWindow *window)
+			     GossipChatWindow *window)
 {
 	GossipChatWindowPriv *priv;
 	
@@ -919,6 +914,8 @@ chat_window_delete_event_cb (GtkWidget        *dialog,
 {
 	GList *list, *l;
 
+	d(g_print ("Chat Window: Delete event received\n"));
+
 	list = g_list_copy (window->priv->chats);
 
 	for (l = list; l; l = l->next) {
@@ -1072,6 +1069,8 @@ chat_window_tab_added_cb (GossipNotebook   *notebook,
 
 	priv = window->priv;
 
+	d(g_print ("Chat Window: Tab added\n"));
+
 	chat = g_object_get_data (G_OBJECT (child), "chat");
 
 	gossip_chat_set_window (chat, window);
@@ -1111,6 +1110,8 @@ chat_window_tab_removed_cb (GossipNotebook   *notebook,
 {
 	GossipChat *chat;
 
+	d(g_print ("Chat Window: Tab removed\n"));
+
 	chat = g_object_get_data (G_OBJECT (child), "chat");
 	
 	gossip_chat_set_window (chat, NULL);
@@ -1146,6 +1147,8 @@ chat_window_tab_detached_cb (GossipNotebook   *notebook,
 	GossipChat	 *chat;
 	gint              x, y;
 
+	d(g_print ("Chat Window: Tab detached\n"));
+
 	chat = g_object_get_data (G_OBJECT (child), "chat");
 	new_window = gossip_chat_window_new ();
 
@@ -1189,22 +1192,24 @@ chat_window_drag_data_received (GtkWidget        *widget,
 				guint             time, 
 				GossipChatWindow *window)
 {
-	GossipContact    *contact;
-	GossipChat       *chat;
-	GossipChatWindow *old_window;
-	const gchar      *id;
+	GossipChatManager *manager;
+	GossipContact     *contact;
+	GossipChat        *chat;
+	GossipChatWindow  *old_window;
+	const gchar       *id;
 
 	id = (const gchar*) selection->data;
-	g_print ("Received drag & drop contact from roster with id:'%s'\n", id);
+	d(g_print ("Chat Window: Received drag & drop contact from roster with id:'%s'\n", id));
 
 	contact = gossip_session_find_contact (gossip_app_get_session (), id);
 	
 	if (!contact) {
-		g_print ("No contact found associated with drag & drop\n");
+		d(g_print ("Chat Window: No contact found associated with drag & drop\n"));
 		return;
 	}
 
-	chat = GOSSIP_CHAT (gossip_chat_manager_get_chat (gossip_app_get_chat_manager (), contact));
+	manager = gossip_app_get_chat_manager ();
+	chat = GOSSIP_CHAT (gossip_chat_manager_get_chat (manager, contact));
 	old_window = gossip_chat_get_window (chat);
 	
 	if (old_window) {
@@ -1218,6 +1223,9 @@ chat_window_drag_data_received (GtkWidget        *widget,
 
 	gossip_chat_window_add_chat (window, chat);
 
+	/* added to take care of any outstanding chat events */
+	gossip_chat_manager_show_chat (manager, contact);
+	
 	gtk_drag_finish (context, TRUE, FALSE, GDK_CURRENT_TIME);
 }
 
@@ -1253,6 +1261,8 @@ gossip_chat_window_add_chat (GossipChatWindow *window,
 {
 	GtkWidget *label;
 
+	d(g_print ("Chat Window: Adding chat\n"));
+
 	label = chat_window_create_label (window, chat);
 
 	if (g_list_length (window->priv->chats) == 0) {
@@ -1277,6 +1287,8 @@ void
 gossip_chat_window_remove_chat (GossipChatWindow *window,
 				GossipChat	 *chat)
 {
+	d(g_print ("Chat Window: Removing chat\n"));
+
 	gossip_notebook_remove_page (GOSSIP_NOTEBOOK (window->priv->notebook),
 				     gossip_chat_get_widget (chat));
 	/* FIXME: somewhat ugly */
