@@ -32,13 +32,11 @@
 #include "gossip-spell.h"
 #include "gossip-spell-dialog.h"
 #include "gossip-ui-utils.h"
+#include "gossip-preferences.h"
 #include "gossip-chat.h"
+#include "gossip-app.h"
 
-#define d(x) x
-
-
-extern GConfClient *gconf_client;
-
+#define d(x)
 
 struct _GossipChatPriv {
 	GossipChatWindow *window;
@@ -155,17 +153,20 @@ gossip_chat_init (GossipChat *chat)
 	GossipChatPriv *priv;
 	GtkTextBuffer  *buffer;
 
+	gchar          *value;
+	GList          *languages = NULL;
+
 	chat->view = gossip_chat_view_new ();
 	chat->input_text_view = gtk_text_view_new ();
 	chat->is_first_char = TRUE;
 
-	g_object_set (G_OBJECT (chat->input_text_view),
+	g_object_set (chat->input_text_view,
 		      "pixels-above-lines", 2,
 		      "pixels-below-lines", 2,
 		      "pixels-inside-wrap", 1,
 		      "right-margin", 2,
 		      "left-margin", 2,
-		      "wrap-mode", GTK_WRAP_WORD,
+		      "wrap-mode", GTK_WRAP_WORD_CHAR,
 		      NULL);
 
 	priv = g_new0 (GossipChatPriv, 1);
@@ -189,15 +190,39 @@ gossip_chat_init (GossipChat *chat)
 			  G_CALLBACK (chat_text_populate_popup_cb),
 			  chat);
 
-	gossip_chat_view_set_margin (chat->view, 3);
-
 	/* create misspelt words identification tag */
 	gtk_text_buffer_create_tag (buffer,
 				    "misspelled",
 				    "underline", PANGO_UNDERLINE_ERROR,
 				    NULL);
 
-	priv->spell = gossip_spell_new (NULL);
+	/* get spelling languages */
+	value = gconf_client_get_string (gossip_app_get_gconf_client (),
+					 GCONF_PATH "/conversation/spell_checker_languages", 
+					 NULL);
+	
+	if (value) {
+		gchar **vlanguages;
+		gchar  *lang;
+		gint    i;
+
+		vlanguages = g_strsplit (value, ",", -1);
+		
+		for (i = 0, lang = vlanguages[i]; lang; lang = vlanguages[++i]) {
+			languages = g_list_append (languages, g_strdup (lang));
+		}
+
+		g_strfreev (vlanguages);
+	}
+	
+	g_free (value);
+	
+	priv->spell = gossip_spell_new (languages);
+	
+	if (languages) {
+		g_list_foreach (languages, (GFunc)g_free, NULL);
+		g_list_free (languages);
+	}
 }
 
 static void
@@ -211,8 +236,10 @@ chat_finalize (GObject *object)
 	chat = GOSSIP_CHAT (object);
 	priv = chat->priv;
 
-	gossip_spell_unref (priv->spell);
-	g_free (priv);
+	if (priv->spell) {
+		gossip_spell_unref (priv->spell);
+		g_free (priv);
+	}
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -228,7 +255,7 @@ chat_input_text_buffer_changed_cb (GtkTextBuffer *buffer, GossipChat *chat)
 
 	priv = chat->priv;
 
- 	spell_checker = gconf_client_get_bool (gconf_client, 
+ 	spell_checker = gconf_client_get_bool (gossip_app_get_gconf_client (), 
 					       "/apps/gossip/conversation/enable_spell_checker", 
 					       NULL); 
 
@@ -557,6 +584,18 @@ gossip_chat_get_contact (GossipChat *chat)
 	return NULL;
 }
 
+GossipContact *
+gossip_chat_get_own_contact (GossipChat *chat)
+{
+	g_return_val_if_fail (GOSSIP_IS_CHAT (chat), NULL);
+
+	if (GOSSIP_CHAT_GET_CLASS (chat)->get_own_contact) {
+		return GOSSIP_CHAT_GET_CLASS (chat)->get_own_contact (chat);
+	}
+
+	return NULL;
+}
+
 void
 gossip_chat_get_geometry (GossipChat *chat,
 		          int        *width,
@@ -714,7 +753,7 @@ gossip_chat_present (GossipChat *chat)
 
 		/* get the default window for either group chats or
 		   normal chats, we do not want to mix them */
-		window = gossip_chat_window_get_default (for_group_chat);
+		window = gossip_chat_window_get_default ();
 
 		if (!window) {
 			window = gossip_chat_window_new ();
@@ -724,9 +763,96 @@ gossip_chat_present (GossipChat *chat)
         }
 
         gossip_chat_window_switch_to_chat (priv->window, chat);
-	gossip_ui_utils_window_present (
+	gossip_window_present (
 		GTK_WINDOW (gossip_chat_window_get_dialog (priv->window)));
 
 	gtk_widget_grab_focus (chat->input_text_view);
 }
 
+gboolean
+gossip_chat_should_play_sound (GossipChat *chat)
+{
+	GossipChatWindow *window;
+	gboolean          play = TRUE;
+
+	g_return_val_if_fail (GOSSIP_IS_CHAT (chat), FALSE);
+
+	window = gossip_chat_get_window (GOSSIP_CHAT (chat));
+	if (!window) {
+		return TRUE; 
+	}
+
+	play = !gossip_chat_window_has_focus (window);
+
+	return play;
+}
+
+gboolean
+gossip_chat_should_highlight_nick (GossipMessage *message,
+				   GossipContact *my_contact)
+{
+	GossipContact *contact;
+	const gchar   *msg, *to;
+	gchar         *cf_msg, *cf_to;
+	gchar         *ch;
+	gboolean       ret_val;
+
+	d(g_print ("ChatView: Highlighting nickname\n"));
+
+	ret_val = FALSE;
+
+	g_return_val_if_fail (GOSSIP_IS_MESSAGE (message), FALSE);
+
+	msg = gossip_message_get_body (message);
+	if (!msg) {
+		return FALSE;
+	}
+		
+	if (my_contact) {
+		contact = my_contact;
+	} else {
+		contact = gossip_message_get_recipient (message);
+	}
+
+	to = gossip_contact_get_name (contact);
+	if (!to) {
+		return FALSE;
+	}
+
+	cf_msg = g_utf8_casefold (msg, -1);
+	cf_to = g_utf8_casefold (to, -1);
+
+	ch = strstr (cf_msg, cf_to);
+	if (ch == NULL) {
+		goto finished;
+	}
+
+	if (ch != cf_msg) {
+		/* Not first in the message */
+		if ((*(ch - 1) != ' ') &&
+		    (*(ch - 1) != ',') &&
+		    (*(ch - 1) != '.')) {
+			goto finished;
+		}
+	}
+
+	ch = ch + g_utf8_strlen (cf_to, -1);
+	if (ch >= cf_msg + g_utf8_strlen (cf_msg, -1)) {
+		ret_val = TRUE;
+		goto finished;
+	}
+
+	if ((*ch == ' ') ||
+	    (*ch == ',') ||
+	    (*ch == '.') ||
+	    (*ch == ':')) {
+		ret_val = TRUE;
+		goto finished;
+	}
+
+finished:
+	g_free (cf_msg);
+	g_free (cf_to);
+
+	return ret_val;
+}
