@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2003-2004 Imendio HB
+ * Copyright (C) 2003-2005 Imendio AB
  * Copyright (C) 2003      Johan Wallenborg <johan.wallenborg@fishpins.se>
  *
  * This program is free software; you can redistribute it and/or
@@ -20,22 +20,26 @@
  */
 
 #include <config.h>
+
 #include <stdio.h>
 #include <unistd.h>
 #include <time.h>
 #include <string.h>
 #include <sys/stat.h>
 #include <sys/types.h>
-#include <libgnome/gnome-i18n.h>
+
+#include <gtk/gtk.h>
+#include <glib/gi18n.h>
 #include <libgnome/gnome-url.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <libxslt/xslt.h>
 #include <libxslt/transform.h>
 #include <libxslt/xsltutils.h>
 #include "libexslt/exslt.h"
-#include "gossip-utils.h"
+
+#include <libgossip/gossip-utils.h>
+
 #include "gossip-app.h"
-#include "gossip-roster.h"
 #include "gossip-log.h"
 
 #define LOG_HEADER \
@@ -50,20 +54,20 @@ static void
 log_ensure_dir (void)
 {
 	gchar *dir;
-	
+
 	dir = g_build_filename (g_get_home_dir (), ".gnome2", NULL);
 	if (!g_file_test (dir, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR)) {
 		mkdir (dir, S_IRUSR | S_IWUSR | S_IXUSR);
 	}
 	g_free (dir);
-	
-	dir = g_build_filename (g_get_home_dir (), ".gnome2", "Gossip", NULL);
+
+	dir = g_build_filename (g_get_home_dir (), ".gnome2", PACKAGE_NAME, NULL);
 	if (!g_file_test (dir, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR)) {
 		mkdir (dir, S_IRUSR | S_IWUSR | S_IXUSR);
 	}
 	g_free (dir);
 
-	dir = g_build_filename (g_get_home_dir (), ".gnome2", "Gossip", "logs", NULL);
+	dir = g_build_filename (g_get_home_dir (), ".gnome2", PACKAGE_NAME, "logs", NULL);
 	if (!g_file_test (dir, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR)) {
 		mkdir (dir, S_IRUSR | S_IWUSR | S_IXUSR);
 	}
@@ -71,20 +75,21 @@ log_ensure_dir (void)
 }
 
 static gchar *
-log_get_filename (GossipJID *jid, const gchar *suffix)
+log_get_filename (GossipContact *contact, 
+		  const gchar   *suffix)
 {
 	gchar *tmp;
 	gchar *ret;
 	gchar *case_folded_str;
 	gchar *escaped_str;
 
-	case_folded_str = g_utf8_casefold (gossip_jid_get_without_resource (jid), 
+	case_folded_str = g_utf8_casefold (gossip_contact_get_id (contact),
 					   -1);
 	escaped_str = gnome_vfs_escape_host_and_path_string (case_folded_str);
 	g_free (case_folded_str);
-	
+
 	tmp = g_build_filename (g_get_home_dir (),
-				".gnome2", "Gossip", "logs",
+				".gnome2", PACKAGE_NAME, "logs",
 				escaped_str,
 				NULL);
 	g_free (escaped_str);
@@ -96,25 +101,12 @@ log_get_filename (GossipJID *jid, const gchar *suffix)
 }
 
 static gchar *
-log_get_timestamp (LmMessage *msg)
+log_get_timestamp (GossipMessage *msg)
 {
-	const gchar *stamp;
-	time_t       t;
-	struct tm   *tm;
-	gchar        buf[128];
-	
-	stamp = gossip_utils_get_timestamp_from_message (msg);
-	if (stamp) {
-		tm = lm_utils_get_localtime (stamp);
-	} else {
-		t  = time (NULL);
-		tm = localtime (&t);
-	}
-	
-	buf[0] = 0;
-	strftime (buf, sizeof (buf), "%Y%m%dT%H:%M:%S", tm);
+	gossip_time_t t;
 
-	return g_strdup (buf);
+	t = gossip_message_get_timestamp (msg);
+	return gossip_time_to_timestamp_full (t, "%Y%m%dT%H:%M:%S");
 }
 
 static gchar *
@@ -126,10 +118,10 @@ log_urlify (const gchar *msg)
 	gchar   *esc;
 
 	ret = g_string_new (NULL);
-	
+
 	start = g_array_new (FALSE, FALSE, sizeof (gint));
 	end = g_array_new (FALSE, FALSE, sizeof (gint));
-	
+
 	num_matches = gossip_utils_url_regex_match (msg, start, end);
 
 	if (num_matches == 0) {
@@ -169,7 +161,7 @@ log_urlify (const gchar *msg)
 			g_free (esc);
 
 			g_string_append (ret, "</a>");
-			
+
 			last = e;
 
 			g_free (tmp);
@@ -186,41 +178,34 @@ log_urlify (const gchar *msg)
 
 	g_array_free (start, TRUE);
 	g_array_free (end, TRUE);
-	
+
 	return g_string_free (ret, FALSE);
 }
 
 void
-gossip_log_message (LmMessage *msg, gboolean incoming)
+gossip_log_message (GossipContact *own_contact,
+		    GossipMessage *msg, 
+		    gboolean       incoming)
 {
-	const gchar   *jid_string;
-        GossipJID     *jid;
+	GossipContact *contact;
         gchar         *filename;
 	FILE          *file;
 	gchar         *body;
 	const gchar   *to_or_from;
 	gchar         *stamp;
-	LmMessageNode *body_node;
-	gchar         *nick;
+	const gchar   *nick;
 	const gchar   *resource;
-	
-	body_node = lm_message_node_get_child (msg->node, "body");
-	if (!body_node) {
-		return;
-	}
-	
+
 	if (incoming) {
-		jid_string = lm_message_node_get_attribute (msg->node, "from");
+		contact = gossip_message_get_sender (msg);
 	} else {
-		jid_string = lm_message_node_get_attribute (msg->node, "to");
+		contact = gossip_message_get_recipient (msg);
 	}
 
-	jid = gossip_jid_new (jid_string);
-	
-	filename = log_get_filename (jid, ".log");
+	filename = log_get_filename (contact, ".log");
 
 	log_ensure_dir ();
-	
+
 	if (!g_file_test (filename, G_FILE_TEST_EXISTS)) {
 		file = fopen (filename, "w+");
 		if (file) {
@@ -230,42 +215,32 @@ gossip_log_message (LmMessage *msg, gboolean incoming)
 		file = fopen (filename, "r+");
 		if (file) {
 			fseek (file, - strlen (LOG_FOOTER), SEEK_END);
-		} 
+		}
 	}
-	
+
 	g_free (filename);
-	
-        if (!file) {
-		gossip_jid_unref (jid);
+
+	if (!file) {
 		return;
 	}
-	
+
 	stamp = log_get_timestamp (msg);
 
 	if (incoming) {
-		GossipRosterItem *item;
-
-		item = gossip_roster_get_item (gossip_app_get_roster (), jid);
-		if (!item) {
-			nick = gossip_jid_get_part_name (jid);
-		} else {
-			nick = g_strdup (gossip_roster_item_get_name (item));
-		}
-		
 		to_or_from = "from";
+		nick = gossip_contact_get_name (contact);
 	} else {
-		nick = gossip_jid_get_part_name (gossip_app_get_jid ());
-
 		to_or_from = "to";
+ 		nick = gossip_contact_get_name (own_contact);
 	}
 
-	if (body_node->value) {
-		body = log_urlify (body_node->value);
+        if (gossip_message_get_body (msg)) {
+		body = log_urlify (gossip_message_get_body (msg));
 	} else {
 		body = g_strdup ("");
 	}
-	
-	resource = gossip_jid_get_resource (jid);
+
+	resource = gossip_message_get_explicit_resource (msg);
 	if (!resource) {
 		resource = "";
 	}
@@ -276,21 +251,20 @@ gossip_log_message (LmMessage *msg, gboolean incoming)
 		 "  </message>\n"
 		 LOG_FOOTER,
 		 stamp, to_or_from,
-		 gossip_jid_get_without_resource (jid),
+		 gossip_contact_get_id (contact),
 		 resource,
 		 nick,
 		 body);
-	
+
         fclose (file);
 
-	gossip_jid_unref (jid);
 	g_free (stamp);
-	g_free (nick);
 	g_free (body);
 }
 
 static gboolean
-log_transform (const gchar *infile, gint fd_outfile)
+log_transform (const gchar *infile, 
+	       gint         fd_outfile)
 {
         xsltStylesheet  *stylesheet;
         xmlDoc          *xml_doc;
@@ -303,7 +277,7 @@ log_transform (const gchar *infile, gint fd_outfile)
 	xmlLoadExtDtdDefaultValue = 1;
 	exsltRegisterAll ();
 
-        stylesheet = xsltParseStylesheetFile (STYLESHEETDIR "/gossip-log.xsl");
+        stylesheet = xsltParseStylesheetFile ((const xmlChar *)STYLESHEETDIR "/gossip-log.xsl");
 	if (!stylesheet) {
 		return FALSE;
 	}
@@ -314,51 +288,129 @@ log_transform (const gchar *infile, gint fd_outfile)
 	}
 
 	title = g_strconcat ("\"", _("Conversation Log"), "\"", NULL);
-	
+
 	/* Set params to be passed to stylesheet. */
 	params[0] = "title";
 	params[1] = title;
         params[2] = NULL;
-	
+
         html_doc = xsltApplyStylesheet (stylesheet, xml_doc, params);
 	if (!html_doc) {
 		xmlFreeDoc (xml_doc);
 		return FALSE;
 	}
-                                                                                
+
         xmlFreeDoc (xml_doc);
-	
+
 	xsltSaveResultToFd (fd_outfile, html_doc, stylesheet);
 
 	xsltFreeStylesheet (stylesheet);
         xmlFreeDoc (html_doc);
 
 	g_free (title);
-	
+
 	return TRUE;
 }
 
 gboolean
-gossip_log_exists (GossipJID *jid)
+gossip_log_exists (GossipContact *contact)
 {
 	gchar    *infile;
 	gboolean  exists;
 
-	infile = log_get_filename (jid, ".log");
+	infile = log_get_filename (contact, ".log");
 	exists = g_file_test (infile, G_FILE_TEST_EXISTS);
+
 	g_free (infile);
 
 	return exists;
-}	
+}
 
-void
-gossip_log_show (GossipJID *jid)
+/* Copyright (C) 1991, 1992, 1996, 1998 Free Software Foundation, Inc.
+ * This file is derived from mkstemp.c from the GNU C Library.
+ */
+
+#include <sys/types.h>
+#include <stdlib.h>
+#include <string.h>
+#include <errno.h>
+#include <stdio.h>
+#include <fcntl.h>
+#include <unistd.h>
+#include <sys/time.h>
+
+#ifndef TMP_MAX
+#define TMP_MAX 16384
+#endif
+
+static int
+our_mkstemps (char *template, int suffix_len)
+{
+	static const char letters[]
+		= "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
+	static guint64 value;
+	struct timeval tv;
+	char *XXXXXX;
+	size_t len;
+	int count;
+
+	len = strlen (template);
+
+	if ((int) len < 6 + suffix_len
+	    || strncmp (&template[len - 6 - suffix_len], "XXXXXX", 6))
+	{
+		return -1;
+	}
+
+	XXXXXX = &template[len - 6 - suffix_len];
+
+	/* Get some more or less random data.  */
+	gettimeofday (&tv, NULL);
+	value += ((guint64) tv.tv_usec << 16) ^ tv.tv_sec ^ getpid ();
+
+	for (count = 0; count < TMP_MAX; ++count)
+	{
+		guint64 v = value;
+		int fd;
+
+		/* Fill in the random bits.  */
+		XXXXXX[0] = letters[v % 62];
+		v /= 62;
+		XXXXXX[1] = letters[v % 62];
+		v /= 62;
+		XXXXXX[2] = letters[v % 62];
+		v /= 62;
+		XXXXXX[3] = letters[v % 62];
+		v /= 62;
+		XXXXXX[4] = letters[v % 62];
+		v /= 62;
+		XXXXXX[5] = letters[v % 62];
+
+		fd = open (template, O_RDWR|O_CREAT|O_EXCL, 0600);
+		if (fd >= 0)
+			/* The file does not exist.  */
+			return fd;
+
+		/* This is a random value.  It is only necessary that the next
+		   TMP_MAX values generated by adding 7777 to VALUE are different
+		   with (module 2^32).  */
+		value += 7777;
+	}
+
+	/* We return the null string if we can't find a unique file name.  */
+	template[0] = '\0';
+
+	return -1;
+}
+
+static void
+log_show (GossipContact *contact)
 {
 	gchar *infile, *outfile, *url;
 	FILE  *file;
 	int    fd;
 
-	infile = log_get_filename (jid, ".log");
+	infile = log_get_filename (contact, ".log");
 
 	if (!g_file_test (infile, G_FILE_TEST_EXISTS)) {
 		file = fopen (infile, "w+");
@@ -373,24 +425,26 @@ gossip_log_show (GossipJID *jid)
 		return;
 	}
 
-	outfile = g_build_filename (g_get_tmp_dir (), "gossip-log-XXXXXX", NULL);
+	outfile = g_build_filename (g_get_tmp_dir (),
+				    "gossip-log-XXXXXX.html",
+				    NULL);
 
-	fd = mkstemp (outfile);
+	fd = our_mkstemps (outfile, 5);
 	if (fd == -1) {
 		g_free (outfile);
 		return;
 	}
-	
+
 	if (!log_transform (infile, fd)) {
 		/* Error dialog... */
-		
+
 		g_free (infile);
 		g_free (outfile);
 		return;
 	}
 
 	close (fd);
-	
+
 	url = g_strconcat ("file://", outfile, NULL);
 	if (!gnome_url_show (url, NULL)) {
 		/* Error dialog... */
@@ -399,3 +453,24 @@ gossip_log_show (GossipJID *jid)
 	g_free (url);
 	g_free (outfile);
 }
+
+void
+gossip_log_show (GtkWidget *window, GossipContact *contact)
+{
+	GdkCursor *cursor;
+
+	cursor = gdk_cursor_new (GDK_WATCH);
+
+	gdk_window_set_cursor (window->window, cursor);
+	gdk_cursor_unref (cursor);
+
+	/* Let the UI redraw before we start the slow transformation. */
+	while (gtk_events_pending ()) {
+		gtk_main_iteration ();
+	}
+
+	log_show (contact);
+
+	gdk_window_set_cursor (window->window, NULL);
+}
+
