@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2003 Imendio HB
+ * Copyright (C) 2003-2005 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,15 +21,14 @@
 #ifndef __GOSSIP_LOG_H__
 #define __GOSSIP_LOG_H__
 
-#include <loudmouth/loudmouth.h>
-#include "gossip-jid.h"
-
-void     gossip_log_message (LmMessage *msg,
-			     gboolean   incoming);
-
-gboolean gossip_log_exists  (GossipJID *jid);
-
-void     gossip_log_show    (GossipJID *jid);
+#include <glib.h>
+#include <libgossip/gossip-message.h>
 
+void     gossip_log_message (GossipContact *own_contact,
+			     GossipMessage *msg,
+			     gboolean       incoming);
+gboolean gossip_log_exists  (GossipContact *contact);
+void     gossip_log_show    (GtkWidget     *window,
+			     GossipContact *contact);
 
 #endif /* __GOSSIP_LOG_H__ */
