@@ -19,7 +19,6 @@
  */
 
 #include <config.h>
-#include <gconf/gconf-client.h>
 #include <libgnome/gnome-sound.h>
 #include <libgnome/gnome-triggers.h>
 
@@ -31,33 +30,42 @@
 
 #define d(x)
 
-extern GConfClient *gconf_client;
+/* is sound toggled on or off */
+static gboolean sound_enabled = TRUE;
+
 
 void
 gossip_sound_play (GossipSound sound)
 {
-	gboolean             enabled, silent_busy, silent_away;
+	GossipSession       *session;
         GossipPresence      *p;
         GossipPresenceState  state;
+	gboolean             enabled, silent_busy, silent_away;
 
-	if (!gossip_session_is_connected (gossip_app_get_session ())) {
+	session = gossip_app_get_session ();
+
+	/* this is the internal sound enable/disable for when events
+	   happen that we need to mute sound - e.g. changing roster contacts */
+	if (!sound_enabled) {
+		d(g_print ("Sound: Play request ignored, sound currently disabled.\n"));
 		return;
 	}
-	
-	enabled = gconf_client_get_bool (gconf_client,
+
+	enabled = gconf_client_get_bool (gossip_app_get_gconf_client (),
 					 GCONF_PATH "/sound/play_sounds",
 					 NULL);
-	silent_busy = gconf_client_get_bool (gconf_client,
-					     GCONF_PATH "/sound/silent_busy",
-					     NULL);
-	silent_away = gconf_client_get_bool (gconf_client,
-					     GCONF_PATH "/sound/silent_away",
-					     NULL);
-
 	if (!enabled) {
+		d(g_print ("Sound: Preferences have sound disabled.\n"));
 		return;
 	}
 
+	silent_busy = gconf_client_get_bool (gossip_app_get_gconf_client (),
+					     GCONF_PATH "/sound/silent_busy",
+					     NULL);
+	silent_away = gconf_client_get_bool (gossip_app_get_gconf_client (),
+					     GCONF_PATH "/sound/silent_away",
+					     NULL);
+
         p = gossip_session_get_presence (gossip_app_get_session ());
         state = gossip_presence_get_state (p);
 
@@ -65,7 +73,8 @@ gossip_sound_play (GossipSound sound)
 		return;
 	}
 
-	if (silent_away && (state == GOSSIP_PRESENCE_STATE_AWAY || state == GOSSIP_PRESENCE_STATE_EXT_AWAY)) {
+	if (silent_away && (state == GOSSIP_PRESENCE_STATE_AWAY || 
+			    state == GOSSIP_PRESENCE_STATE_EXT_AWAY)) {
 		return;
 	}
 
@@ -88,3 +97,7 @@ gossip_sound_play (GossipSound sound)
 	}
 }		
 
+void gossip_sound_toggle (gboolean enabled)
+{
+	sound_enabled = enabled;
+}
